package com.jsl.oa.controllers;

import com.jsl.oa.model.doData.MessageAddDO;
import com.jsl.oa.services.MessageService;
import com.jsl.oa.utils.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.naming.Binding;
import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.LocalDateTime;

/**
 * <h1>消息控制器</h1>
 * <hr/>
 * 消息控制器，包含信息获取接口
 *
 * @author 张睿相
 * @version v1.1.0
 * @since v1.1.0
 */
@Slf4j
@RestController
@RequiredArgsConstructor
public class MessageController {

    private final MessageService messageService;

    @DeleteMapping("/message/delete")
    public BaseResponse messageDelete(@RequestParam Long id, HttpServletRequest request) {
        log.info("请求接口[DELETE]:/message/delete");
        if (id == null) {
            return ResultUtil.error(ErrorCode.PARAMETER_ERROR);
        } else return messageService.messageDelete(id, request);
    }

    @GetMapping("/message/get")
    public BaseResponse messageGet(@RequestParam(required = false,defaultValue = "1") Integer page,
                                   @RequestParam(required = false,defaultValue = "10") Integer pageSize,
                                   HttpServletRequest request,
                                   @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate begin,
                                   @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate end) {
        log.info("请求接口[GET]:/message/get");
        String token = request.getHeader("Authorization").replace("Bearer ", "");
        Long uid = JwtUtil.getUserId(token);
        if (uid == null) {
            return ResultUtil.error(ErrorCode.USER_NOT_EXIST);
        } else return messageService.messageGet(begin, end, page, pageSize, uid);
    }

    /*
    @GetMapping("/message/get/all")
    public BaseResponse messageGetAll(@RequestParam(defaultValue = "1") Integer page,
                                      @RequestParam(defaultValue = "10") Integer pageSize,
                                      HttpServletRequest request,
                                      @RequestParam Long uid,
                                      @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate begin,
                                      @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate end) {
        log.info("请求接口[GET]:/message/get/all");
        String token = request.getHeader("Authorization").replace("Bearer ", "");
        Long LoginId = JwtUtil.getUserId(token);
        if (LoginId == null) {
            return ResultUtil.error(ErrorCode.USER_NOT_EXIST);
        } else return messageService.messageGetAll(begin, end, page, pageSize, LoginId, uid);
    }
*/
    @PostMapping("/message/add")
    public BaseResponse messageAdd(HttpServletRequest request,
                                   @RequestBody @Validated MessageAddDO messageAddDO,
                                   @NotNull BindingResult bindingResult) {
        log.info("请求接口[POST]:/message/add");
        //校验参数
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(ErrorCode.REQUEST_BODY_ERROR, Processing.getValidatedErrorList(bindingResult));
        }
        return messageService.messageAdd(request, messageAddDO);
    }

}


