package com.jsl.oa.services.impl;


import com.jsl.oa.annotations.CheckUserAbleToUse;
import com.jsl.oa.annotations.CheckUserHasPermission;
import com.jsl.oa.mapper.MessageMapper;
import com.jsl.oa.mapper.RoleMapper;
import com.jsl.oa.mapper.UserMapper;
import com.jsl.oa.model.doData.MessageAddDO;
import com.jsl.oa.model.doData.MessageDO;
import com.jsl.oa.model.doData.PageBeanDO;
import com.jsl.oa.model.voData.MessageGetVO;
import com.jsl.oa.model.voData.ProjectSimpleVO;
import com.jsl.oa.services.MessageService;
import com.jsl.oa.utils.BaseResponse;
import com.jsl.oa.utils.ErrorCode;
import com.jsl.oa.utils.JwtUtil;
import com.jsl.oa.utils.ResultUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;


/**
 * <h1>消息服务层实现类</h1>
 * <hr/>
 * 用于消息服务层的实现类
 *
 * @author xiao_lfeng | xiangZr-hhh | 176yunxuan
 * @version v1.1.0
 * @since v1.1.0
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class MessageServiceImpl implements MessageService {

    private final MessageMapper messageMapper;
    private final RoleMapper roleMapper;
    private final UserMapper userMapper;

    @Override
    //@CheckUserHasPermission("message.delete")
    public BaseResponse messageDelete(Long mid, HttpServletRequest request) {
        log.info("\t执行 service 层 messageDelete 方法");
        //获取消息数据
        MessageDO messageDO = messageMapper.getMessageById(mid);
        //检测用户id与消息的uid是否相同
        String token = request.getHeader("Authorization").replace("Bearer ", "");
        if (!(messageDO.getUid().equals(JwtUtil.getUserId(token)))) {
            return ResultUtil.error(ErrorCode.MESSAGE_ONLY_DELETE_BY_THEMSELVES);
        }
        //执行删除
        if (!messageMapper.deleteMessage(mid)) {
            return ResultUtil.error(ErrorCode.DATABASE_UPDATE_ERROR);
        }
        return ResultUtil.success();
    }

    @Override
    //@CheckUserAbleToUse
    public BaseResponse messageGet(LocalDate begin, LocalDate end, Integer page,
                                   Integer pageSize, Long uid) {
        log.info("\t执行 service 层 messageGet 方法");

        //默认获取时间为最近30天
        if (begin == null) {
            end = LocalDate.now();
            begin = end.minusDays(30);
        }
        List<MessageDO> messageList = messageMapper.page(begin, end, uid);
        List<MessageGetVO> messageGetVOList = new ArrayList<>();
        for (MessageDO messageDO : messageList) {
            MessageGetVO messageGetVO = new MessageGetVO();
            messageGetVO.setSidName(userMapper.getUserById(messageDO.getSid()).getUsername());
            messageGetVO.setTxt(messageDO.getText());
            messageGetVO.setTitle(messageDO.getTitle());
            messageGetVO.setCreatedAt(messageDO.getCreatedAt());
            messageGetVOList.add(messageGetVO);
        }

        //3.分页返回
        int start = (page-1) * pageSize;
        int endIndex = start + pageSize;
        List<MessageGetVO> pageData = messageGetVOList.subList(start,
                Math.min(endIndex,messageGetVOList.size()));
        return ResultUtil.success(pageData);
    }

    /*
    @Override
    @CheckUserHasPermission("message.get")
    public BaseResponse messageGetAll( LocalDate begin, LocalDate end, Integer page,
                                       Integer pageSize, Long loginId, Long uid) {
        log.info("请求接口服务层");
        //1.记录总数据数
        Long count = messageMapper.count(uid);

        //2.获取分页数据列表
        //默认获取时间为最近30天
        if (begin == null) {
            begin = LocalDate.now();
            end = begin.minusDays(30);
        }
        List<MessageDO> messageDOList = messageMapper.page(begin, end, uid);

        //3.分页返回
        int start = (page-1) * pageSize;
        int endIndex = start + pageSize;
        List<MessageDO> pageData = messageDOList.subList((start),
                Math.min(endIndex,messageDOList.size()));
        return ResultUtil.success(pageData);
    }
*/
    @Override
    public BaseResponse messageAdd(HttpServletRequest request, MessageAddDO messageAddDO) {
        log.info("\t执行 service 层 messageAdd 方法");

        Long sid = JwtUtil.getUserId(request.getHeader("Authorization").replace("Bearer ", ""));
        messageAddDO.setSid(sid);
        if(!messageMapper.messageAdd(messageAddDO)){
            return ResultUtil.error(ErrorCode.DATABASE_INSERT_ERROR);
        }
        return ResultUtil.success();
    }


}


